unit ICCheckboxWhats;

(*	This file is part of the Internet Configuration system and is placed in the public domain for the benefit of all.

	This module implements the Checkbox What, a very simple What that just
	loads and saves a Boolean value determined by a checkbox.
*)

interface

	uses
		ICWindowGlobals;

	(* ***** Standard What Interface ***** *)
	
	(* See the comments in ICWindowGlobals. *)

	function WhatOpenCheckbox (wt: WindowType; item: integer): OSStatus;
	function WhatFlushCheckbox (wt: WindowType; item: integer): OSStatus;
	function WhatClickCheckbox (wt: WindowType; item: integer; event: EventRecord): OSStatus;

implementation

	uses
		ICAPI,

		ICDialogs,
		ICWindowGlobals;

	(* ***** Standard What Interface ***** *)

	(* ***** What Open ***** *)

	function WhatOpenCheckbox (wt: WindowType; item: integer): OSStatus;
		(* See the comments in ICWindowGlobals. *)
		var
			checkboxValue : SInt8;
			attr : ICAttr;
			size : longint;
	begin
		(* Read the current setting. *)
		size := sizeof(checkboxValue);
		if ICGetPref(WhatGetDocumentInstance, gWindows[wt].items[item]^.whatKey, attr, @checkboxValue, size) <> noErr then begin
			checkboxValue := 0;
		end; (* if *)
		WhatProcessAttributes(wt, item, attr);
		checkboxValue := band(checkboxValue, 1);
		
		(* Set the checkbox to reflect the current setting and the lock flag. *)
		SetDControlValue(gWindows[wt].window, item, checkboxValue);
		SetDControlEnable(gWindows[wt].window, item, not WhatIsLocked(wt, item));
		gWindows[wt].items[item]^.whatModified := false;
		WhatOpenCheckbox := noErr;
	end; (* WhatOpenCheckbox *)
	
	(* ***** What Flush ***** *)

	function WhatFlushCheckbox (wt: WindowType; item: integer): OSStatus;
		(* See the comments in ICWindowGlobals. *)
		var
			err : OSStatus;
			checkboxValue : SInt8;
	begin
		err := noErr;
		if gWindows[wt].items[item]^.whatModified then begin
			checkboxValue := band(GetDControlValue(gWindows[wt].window, item), 1);
			err := ICSetPref(WhatGetDocumentInstance, gWindows[wt].items[item]^.whatKey, ICattr_no_change, @checkboxValue, 1);
			gWindows[wt].items[item]^.whatModified := false;
		end; (* if *)
		WhatFlushCheckbox := err;
	end; (* WhatFlushCheckbox *)

	(* ***** What Click ***** *)
	
	function WhatClickCheckbox (wt: WindowType; item: integer; event: EventRecord): OSStatus;
		(* See the comments in ICWindowGlobals. *)
	begin
		{$unused event}
		gWindows[wt].items[item]^.whatModified := true;
		ToggleDControlBoolean(gWindows[wt].window, item);
		WhatDirtyDocument;
		WhatClickCheckbox := noErr;
	end; (* WhatClickCheckbox *)

end. (* ICCheckboxWhats *)