unit ICMiscSubs;

(*	This file is part of the Internet Configuration system and is placed in the public domain for the benefit of all.

	This file holds all those miscellaneous little functions that are basically wrappers
	around existing OS functionality.
*)

interface

	uses
		Files,
		Windows,
		Lists, 
		AppleEvents, 

		ICTypes;

	(* ***** Memory Stuff ***** *)
	
	type
		(* A data structure for addresses memory as bytes. *)
		BigBuffer = packed array [0..$0FFFFFF] of Byte;
		BigBufferPtr = ^BigBuffer;
		BigBufferHandle = ^BigBufferPtr;

		(* Another for addressing memory as chars. *)
		BigCharArray = packed array [0..$FFFFFF] of char;
		BigCharArrayPtr = ^BigCharArray;
		BigCharArrayHandle = ^BigCharArrayPtr;

	function BlockCompare (lhsBaseAddr : univ Ptr; rhsBaseAddr: univ Ptr; size: longint): Boolean;
		(* Compares two blocks of memory for equality.*)
		
	procedure BlockFill (baseAddr: univ Ptr; size: longint; value: integer);
		(* Fills a block of memory with the value.  The memory is filled
			as bytes, ie the high byte of value is ignored.
		*)

	(* ***** File Manager Stuff ***** *)
	
	function GetVolInfo (var ioName: Str63; var ioVRefNum: integer; ioVolIndex: integer;
										var ioVCrDate: longint): OSStatus;
		(* Returns information about the specified volume. Basically this is a wrapper
			around PBGetVInfo.  See IM:Files for a description of the meaning of the
			ioName, ioVRefNum and ioVolIndex parameters.  This routine also returns
			the volume's creation date in ioVCrDate to aid in Poor Man's Alias Manager
			volume matching.
		*)
		
	function FindApplicationInDTDB (creator: OSType; var foundApplicationSpec: FSSpec): OSStatus;
		(* This routine attempts to find an application in the desktop database
			given its creator type.
		*)
		
	function FSpGetCatInfo (var fss: FSSpec; ioFDirIndex: integer; var cpb: CInfoPBRec): OSStatus;
		(* This routine is a simple wrapper around PBGetCatInfo.  See IM:Files
			for a description of the meaning of ioFDirIndex.
			Note that, despite the name, this routine can be called under System 6.
		*)
		
	function FSpSetCatInfo (var fss: FSSpec; var cpb: CInfoPBRec): OSStatus;
		(* This routine is a simple wrapper around PBSetCatInfo.
			Note that, despite the name, this routine can be called under System 6.
		*)

	function FileLocked (fss: FSSpec): Boolean;
		(* This routine returns true if the specified file is locked. Note that
			this provides no guarantee that you can write to the file; it merely
			checks all the things it can to see if any of them disable writing.
		*)
		
	function CopyFile (source : FSSpec; dest: FSSpec): OSStatus;
		(* Copies a file from source to dest.  Any file already existing at dest is
			destroyed.  Also set the catalogue info for the dest file from the source file.
		*)

	(* ***** QuickDraw Stuff ***** *)

	procedure DrawIcon (resourceID: integer; iconRect: Rect; drawHighlighted: Boolean);
		(* This routine draws in icon from the resources specified by resourceID.
			If the System 7 icon utilities are available, it uses the icon family
			resources 'icl8', and draws using the icon utilities.  If they're not available,
			it uses the 'ICN#' resource and draws using PlotIcon.
		*)
	
	procedure MagicMarkerMode;
		(* This routine sets the HiliteMode low memory global such that the
			next invert operation is done using the user specified highlight colour.
			If Colour QuickDraw isn't available, it does nothing.
		*)

	(* ***** IC API Stuff ***** *)

	(* These are simple wrappers around the IC API for getting and setting PString preferences. *)

	function ICGetPrefStr (inst: ICInstance; key: Str255; var attr: ICAttr; var str: Str255): ICError;
	function ICSetPrefStr (inst: ICInstance; key: Str255; attr: ICAttr; str: Str255): ICError;

	(* ***** Text Utilities Stuff ***** *)

	(* These are simple wrappers around the toolbox NumToString and StringToNum routines. *)

	function DecStr (aNumber : longint): Str255;
	function DecVal(aString : Str255) : integer;

	(* These are simple routines to convert between strings and OSTypes. *)
	
	function StringToOSType (aString: Str255): OSType;
	function OSTypeToString (anOSType: OSType): Str15;

	function TPCopy (sourceString: string; startIndex : integer; count: integer): string;
		(* TPCopy provides a version of the Pascal built-in function Copy that
			implements the Think Pascal semantics.  This is much more useful
			than routine built-in to Metrowerks Pascal, which implements the
			semantics of MPW Pascal.
			The routine extracts count characters from the source string
			starting at character position start.  If there aren't enough characters
			in the string, it returns what there are.
		*)
		
	function GetOwnerName: Str255;
		(* This function returns the Owner name for the Macintosh
			as defined in the Sharing Setup control panel.
		*)
		
	function NewLookupError(NersID : integer; errNum : OSStatus) : Str255;
	(* Return the string associated with errNum.  The 'Ners' resource
		has a ResEdit template of...
				*****
				LSTB
					errNum
					DLNG
					errstr
					PSTR
				*****
				LSTE
		There must be a terminating entry with error number of 0 that contains
		the default error message.
	*)

	(* ***** Event Manager Stuff ***** *)

	(* A collection of useful ASCII character definitions. *)			
	const
		kNulChar				= chr($00);
		kHomeChar 			= chr($01);
		kEnterChar 			= chr($03);
		kEndChar 				= chr($04);
		kHelpChar 			= chr($05);
		kBackSpaceChar 	= chr($08);
		kTabChar 				= chr($09);
		kLineFeedChar 		= chr($0A);
		kPageUpChar 		= chr($0B);
		kPageDownChar 	= chr($0C);
		kCRChar 				= chr($0D);
		kEscChar 				= chr($1B);
		kClearChar 			= chr($1B);
		kLeftArrowChar 	= chr($1C);
		kRightArrowChar 	= chr($1D);
		kUpArrowChar 		= chr($1E);
		kDownArrowChar	= chr($1F);
		kSpaceChar 			= chr($20);
		kDelChar 				= chr($7F);
		kBulletChar 			= chr($A5);

	(* A collection of useful virtual key code definitions. *)			
	const
		kUndoKeyCode 				= 122;
		kCutKeyCode 				= 120;
		kCopyKeyCode 				= 99;
		kPasteKeyCode 			= 118;
		kClearKeyCode 				= 71;

		kEscKeyCode 				= 53;
		kReturnKeyCode 			= 36;
		kEnterKeyCode 				= 52;
		kTabKeyCode 				= 48;
		kSpaceKeyCode				= 49;
		KDeleteKeyCode 			= 51;

		kCommandKeyCode 		= 55;
		kShiftKeyCode 				= 56;
		kCapsLockKeyCode 		= 57;
		kOptionKeyCode 			= 58;

	function DirtyKey (typedChar: char): Boolean;
		(* This function returns true if the given character will cause a Text
			Edit field to become dirty, ie it's a character that will go into
			the field rather than move the insertion point.
		*)
		
	function IsKeyDown (keyCode: integer): Boolean;
		(* Returns true if the given virtual key is down. *)

	(* ***** Window Manager Stuff ***** *)

	(* EnterWindow, ExitWindow and the SavedWindowState type are used to implement
		a standard mechanism for saving and restoring window information.  You call
		EnterWindow when you want to work on a window.  This sets up the parameters
		you need and saves the old parameters in the SavedWindowState variable.
		You then call ExitWindow to restore that state.
	*)
	
	type
		SavedWindowInfo =
			record
				oldPort: GrafPtr;
				thisPort: GrafPtr;
				font: integer;
				size: integer;
				face: Style;
			end;

	procedure EnterWindow (window: WindowPtr; font, size: integer; face: Style;
												var saved: SavedWindowInfo);
		(* Set thePort to window and establish the various window state parameters.
			Save the old parameters in saved.
		*)
		
	procedure ExitWindow (saved: SavedWindowInfo);
		(* Recover the window parameters from saved. *)

	function GetWindowContentRegion (theWindow: WindowPtr): RgnHandle;
		(* Returns the window's content region. This is the region currently
			being used, not a copy.  Do not munge it!
		*)
		
	function GetWindowStructureRegion (theWindow: WindowPtr): RgnHandle;
		(* Returns the window's structure region. This is the region currently
			being used, not a copy.  Do not munge it!
		*)

	function TitleBarOnScreen (theWindow: WindowPtr): Boolean;
		(* Returns true if the window's title bar is on the screen.
			Note that this routine only works if the window is visible,
			ie you have called ShowWindow on it.  The standard mechanism
			for using this routine is to ShowWindow the window, then
			call TitleBarOnScreen.  If it returns true, everything is cool.
			Otherwise the window is completely off the screen, so you can
			move it back on without causing visible effects.
		*)
		
	procedure GetWindowRect (theWindow: WindowPtr; var windowRect: Rect);
		(* This routine sets windowRect to the global co-ordinates of 
			the position of the window.  It's typically used for saving window
			state.
		*)
		
	(* ***** Menu Manager Stuff ***** *)

	procedure SetMenuItemEnable (menuH: MenuHandle; item: integer; enable: Boolean);
		(* Enable the item in the MenuHandle if enable is set, disable it otherwise.
			You've gotta wonder why this isn't in the operating system!
		*)
		
	procedure SafeAppendMenu (menuH: MenuHandle; itemText: Str255);
		(* The system AppendMenu interprets the new menu item in strange
			ways, attempting to glean command key and other information from 
			the text.  This is obviously bad for things like the items on the 
			Archie menu.  This 'safe' version of the AppendMenu routine
			sets the text window interpreting it.
		*)
		
	function FindMenuItem (menuH: MenuHandle; itemTextToSearchFor: Str255; 
												var indexOfItemFound: integer): Boolean;
		(* This routine searches through the Menu Handle looking for
			itemTextToSearchFor.  If it finds it, it returns true and sets
			indexOfItemFound to the position of the matching menu item.
		*)

	(* ***** List Manager Stuff ***** *)

	(* All of these List Manager routines are really targetted at one dimensional
		vertical lists.  They don't work well for two dimensional or horizontal
		lists.
	*)
	
	procedure InitListManagerMiscSubs;
		(* The LDoKey function requires a bunch of global state to implement
			it's "select by typing" function. This routine initialises that
			information.
		*)
	
	procedure LSetNoSelection (listH: ListHandle);
		(* This routine clears any selection in the list. *)
		
	procedure LSetSingleSelection (listH: ListHandle; row: integer);
		(* This routine selects the single cell (0, row) in the list. *)

	(* The LDoKey routine takes a procedural parameter that is uses to fetch
		the text associated with an item in the list so that it can implement
		its "select by typing" function.
	*)
	type
		GetListCellTextProcType = function(listH: ListHandle; listCell: Cell): Str255;

	procedure LDoKey(listH: ListHandle; var event:EventRecord; getCellText: GetListCellTextProcType);
		(* This routine processes a key event associated with a list, including
			"select by typing".  You can disable this function by passing nil to
			getCellText.
		*)

	function LSelectedLine (lh: ListHandle): integer;
		(* This function returns the vertical position of the first selected
			cell in the list, or -1 if there is no selected cell.
		*)

	(* ***** Truly Misc Stuff ***** *)

	function AEGotRequiredParams (theAppleEvent: AppleEvent): OSStatus;
		(* Returns no error if you've extracted all of the required
			parameters out of the AppleEvent.
		*)
		
	function TrapAvailable (theTrap: integer): Boolean;
		(* Returns true if theTrap is available on this machine.
			This routine is implemented by the book, the book being Inside Macintosh.
		*)

implementation

	uses
		Icons, 
		Errors, 
		Resources, 
		Dialogs, 
		ToolUtils, 
		Traps, 
		LowMem,

		ICAPI,
		
		ICGlobals;

	(* ***** Memory Stuff ***** *)

{$PUSH}
{$R-}
	function BlockCompare (lhsBaseAddr : univ Ptr; rhsBaseAddr: univ Ptr; size: longint): Boolean;
		(* See comment in interface part. *)
	begin
		BlockCompare := false;
		while (size > 0) do begin
			if lhsBaseAddr^ <> rhsBaseAddr^ then begin
				exit(BlockCompare);
			end; (* if *)
			inc(longint(lhsBaseAddr));
			inc(longint(rhsBaseAddr));
			size := size - 1;
		end; (* while *)
		BlockCompare := true;
	end; (* BlockCompare *)

	procedure BlockFill (baseAddr: univ Ptr; size: longint; value: integer);
		(* See comment in interface part. *)
	begin
		while (size > 0) do begin
			baseAddr^ := value;
			inc(longint(baseAddr));
			size := size - 1;
		end; (* while *)
	end; (* BlockFill *)
{$POP}

	(* ***** File Manager Stuff ***** *)

	function GetVolInfo (var ioName: Str63; var ioVRefNum: integer; ioVolIndex: integer;
										var ioVCrDate: longint): OSStatus;
		(* See comment in interface part. *)
		var
			err: OSStatus;
			pb: ParamBlockRec;
	begin
		(* If we're trying to look up a volume by name, make sure there's a colon
			on the end of the name.
		*)
		if (ioName <> '') & (ioName[length(ioName)] <> ':') then begin
			ioName := concat(ioName, ':');
		end; (* if *)
		pb.ioNamePtr := @ioName;
		pb.ioVRefNum := ioVRefNum;
		pb.ioVolIndex := ioVolIndex;
		err := PBGetVInfoSync(@pb);
		if err = noErr then begin
			ioVRefNum := pb.ioVRefNum;
			ioVCrDate := pb.ioVCrDate;
		end; (* if *)
		GetVolInfo := err;
	end; (* GetVolInfo *)

	function FindApplicationInDTDB (creator: OSType; var foundApplicationSpec: FSSpec): OSStatus;
		(* See comment in interface part. *)
		var
			err: OSStatus;
			junkCreationDate: longint;
			volumeIndex: integer;
			pbdt: DTPBRec;
			found: Boolean;
	begin
		found := false;
		if gHasSystem7 then begin
			(* Repeat through each of the volumes in their enumeration order,
				querying the DTDB on each volume.
			*)
			volumeIndex := 1;
			repeat
				foundApplicationSpec.vRefNum := 0;
				foundApplicationSpec.name := '';
				err := GetVolInfo(foundApplicationSpec.name, foundApplicationSpec.vRefNum, volumeIndex, junkCreationDate);

				(* On this volume, attempt to find the application.  First get the path
					for the DTDB.
				*)
				if err = noErr then begin
					foundApplicationSpec.name := '';
					pbdt.ioNamePtr := @foundApplicationSpec.name;
					pbdt.ioVRefNum := foundApplicationSpec.vRefNum;
					err := PBDTGetPath(@pbdt);
					
					(* We have the path for the DTDB, now lookup the application in it. *)
					if err = noErr then begin
						pbdt.ioIndex := 0;
						pbdt.ioFileCreator := creator;
						err := PBDTGetAPPLSync(@pbdt);
						if err = noErr then begin
							found := true;
						end; (* if *)
					end; (* if *)
					
					(* Ignore errors from the DTDB, so we continue on with the next volume. *)
					err := noErr;
				end; (* if *)
				volumeIndex := volumeIndex + 1;
			until found or (err <> noErr);
		end; (* if *)
		
		(* Clean up.  If we found the application, set the parID.  Otherwise return an
			innocuous FSSpec.
		*)
		if found then begin
			err := noErr;
			foundApplicationSpec.parID := pbdt.ioAPPLParID;
		end else begin
			err := afpItemNotFound;
			foundApplicationSpec.vRefNum := 0;
			foundApplicationSpec.parID := 2;
			foundApplicationSpec.name := '';
		end; (* if *)
		FindApplicationInDTDB := err;
	end; (* FindApplicationInDTDB *)

	function FSpGetCatInfo (var fss: FSSpec; ioFDirIndex: integer; var cpb: CInfoPBRec): OSStatus;
		(* See comment in interface part. *)
	begin
		cpb.ioVRefNum := fss.vRefNum;
		cpb.ioDirID := fss.parID;
		cpb.ioNamePtr := @fss.name;
		cpb.ioFDirIndex := ioFDirIndex;
		FSpGetCatInfo := PBGetCatInfoSync(@cpb);
	end; (* FSpGetCatInfo *)

	function FSpSetCatInfo (var fss: FSSpec; var cpb: CInfoPBRec): OSStatus;
		(* See comment in interface part. *)
	begin
		cpb.ioVRefNum := fss.vRefNum;
		cpb.ioDirID := fss.parID;
		cpb.ioNamePtr := @fss.name;
		FSpSetCatInfo := PBSetCatInfoSync(@cpb);
	end; (* FSpSetCatInfo *)

	function IsVolumeWriteable (vRefNum: integer): OSStatus;
		(* This routine returns noErr if the specified volume is writable,
			or an appropriate error otherwise.
		*)
		var
			err: OSStatus;
			pb: HParamBlockRec;
	begin
		pb.ioVRefNum := vRefNum;
		pb.ioNamePtr := nil;
		pb.ioVolIndex := 0;
		err := PBHGetVInfoSync(@pb);

		if err = noErr then begin
			if band(pb.ioVAtrb, $0080) <> 0 then begin
				err := wPrErr;			(* volume locked by hardware *)
			end else if band(pb.ioVAtrb, $8000) <> 0 then begin
				err := vLckdErr;			(* volume locked by software *)
			end; (* if *)
		end; (* if *)

		IsVolumeWriteable := err;
	end; (* IsVolumeWriteable *)

	function IsFileWriteable (fss: FSSpec): OSStatus;
		(* This routine returns noErr if the specified file is writeable,
			or an appropriate error otherwise.
		*)
		var
			err: OSStatus;
			cpb: CInfoPBRec;
	begin
		err := FSpGetCatInfo(fss, 0, cpb);
		if err = noErr then begin
			if band(cpb.ioFlAttrib, $01) <> 0 then begin
				err := fLckdErr;
			end; (* if *)
		end; (* if *)
		IsFileWriteable := err;
	end; (* IsFileWriteable *)

	function HGetDirAccess (ioVRefNum: integer; ioDirID: longint; ioName: StringPtr;
											var ownerID, groupID, accessRights: longint): OSStatus;
		(* This routine returns the directory access privileges for the specified directory. *)
		var
			err: OSStatus;
			pb: HParamBlockRec;
	begin
		pb.ioNamePtr := ioName;
		pb.ioVRefNum := ioVRefNum;
		pb.ioDirID := ioDirID;
		err := PBHGetDirAccessSync(@pb);
		ownerID := pb.ioACOwnerID;
		groupID := pb.ioACGroupID;
		accessRights := pb.ioACAccess;
		HGetDirAccess := err;
	end; (* HGetDirAccess *)

	function FileLocked (fss: FSSpec): Boolean;
		(* See comment in interface part. *)
		var
			locked: Boolean;
			junk: longint;
			access: longint;
	begin
		locked := (IsVolumeWriteable(fss.vRefNum) <> noErr);
		if not locked then begin
			locked := (IsFileWriteable(fss) <> noErr);
		end; (* if *)
		if not locked then begin
			if HGetDirAccess(fss.vRefNum, fss.parID, nil, junk, junk, access) = noErr then begin
				locked := not btst(access, 26);
			end; (* if *)
		end; (* if *)
		FileLocked := locked;
	end; (* FileLocked *)

	function CopyFork (sourceForkRefnum, destForkRefnum: integer; bytesToCopy: longint): OSStatus;
		(* This routine copies a file fork from sourceForkRefnum to destForkRefnum.
			The files must be positioned at where you want to start copying (usually
			at the beginning) and the routine copys bytesToCopy bytes from the source
			to the destination.
		*)
		const
			kMaxCopyBufferSize = 65536;
			kMinCopyBufferSize = 512;
		var
			err: OSStatus;
			copyBuffer: Ptr;
			copyBufferSize: longint;
			numberOfBytesThisTime: longint;
	begin
		(* First attempt to allocate a copy buffer.  We do this by attempting
			to allocate a buffer of size kMaxCopyBufferSize.  If that fails,
			we divide the size by two and try again.  We keep trying until
			the size drops below kMinCopyBufferSize, after which we
			give up and return an error.
		*)
		err := noErr;
		copyBufferSize := kMaxCopyBufferSize;
		copyBuffer := nil;
		repeat
			copyBuffer := NewPtr(copyBufferSize);
			if copyBuffer = nil then begin
				copyBufferSize := copyBufferSize div 2;
			end; (* if *)
		until (copyBuffer <> nil) or (copyBufferSize < kMinCopyBufferSize);
		if copyBuffer = nil then begin
			err := memFullErr;
		end; (* if *)
		
		(* Now copy the file data, in copyBufferSize chunks. *)
		while (err = noErr) & (bytesToCopy > 0) do begin
			numberOfBytesThisTime := copyBufferSize;
			if numberOfBytesThisTime > bytesToCopy then begin
				numberOfBytesThisTime := bytesToCopy;
			end; (* if *)
			err := FSRead(sourceForkRefnum, numberOfBytesThisTime, copyBuffer);
			if err = noErr then begin
				bytesToCopy := bytesToCopy - numberOfBytesThisTime;
				err := FSWrite(destForkRefnum, numberOfBytesThisTime, copyBuffer);
			end; (* if *)
		end; (* while *)
		
		(* Clean up. *)
		if copyBuffer <> nil then begin
			DisposePtr(copyBuffer);
		end; (* if *)
		CopyFork := err;
	end; (* CopyFork *)

	function CopyFile (source : FSSpec; dest: FSSpec): OSStatus;
		(* See comment in interface part. *)
		var
			err : OSStatus;
			err2 : OSStatus;
			junk: OSStatus;
			cpb: CInfoPBRec;
			sourceResourceRefnum : integer;
			sourceDataRefnum : integer;
			destResourceRefnum : integer;
			desetDataRefnum : integer;
	begin
		(* Start off by deleting the destination file. *)
		junk := HDelete(dest.vRefNum, dest.parID, dest.name);
		
		(* Copy both forks of the file. Two guesses who wrote this chunk of code! *)
		err := FSpGetCatInfo(source, 0, cpb);
		if err = noErr then begin
			err := HCreate(dest.vRefNum, dest.parID, dest.name, cpb.ioFlFndrInfo.fdCreator, cpb.ioFlFndrInfo.fdType);
			if err = noErr then begin
				err := HOpen(dest.vRefNum, dest.parID, dest.name, fsWrPerm, desetDataRefnum);
				if err = noErr then begin
					err := HOpenRF(dest.vRefNum, dest.parID, dest.name, fsWrPerm, destResourceRefnum);
					if err = noErr then begin
						err := HOpen(source.vRefNum, source.parID, source.name, fsRdPerm, sourceDataRefnum);
						if err = noErr then begin
							err := HOpenRF(source.vRefNum, source.parID, source.name, fsRdPerm, sourceResourceRefnum);
							if err = noErr then begin
								err := CopyFork(sourceDataRefnum, desetDataRefnum, cpb.ioFlLgLen);
								if err = noErr then begin
									err := CopyFork(sourceResourceRefnum, destResourceRefnum, cpb.ioFlRLgLen);
								end; (* if *)
								junk := FSClose(sourceResourceRefnum);
							end; (* if *)
							junk := FSClose(sourceDataRefnum);
						end; (* if *)
						err2 := FSClose(destResourceRefnum);
						if err = noErr then begin
							err := err2;
						end; (* if *)
					end; (* if *)
					err2 := FSClose(desetDataRefnum);
					if err = noErr then begin
						err := err2;
					end; (* if *)
				end; (* if *)
			end; (* if *)
		end; (* if *)
		
		(* Set the catalogue info for the destination file. *)
		if err = noErr then begin
			err := FSpSetCatInfo(dest, cpb);
		end; (* if *)
		
		(* Clean up.  Delete the file if we didn't succeed completely. *)
		if err <> noErr then begin
			junk := HDelete(dest.vRefNum, dest.parID, dest.name);
		end; (* if *)
		CopyFile := err;
	end; (* CopyFile *)

	(* ***** QuickDraw Stuff ***** *)

	procedure DrawIcon (resourceID: integer; iconRect: Rect; drawHighlighted: Boolean);
		(* See comment in interface part. *)
		var
			junk: OSStatus;
			iconSuite: Handle;
			transform: integer;
			iconH: Handle;
	begin
		if gHasSystem7 & (GetIconSuite(iconSuite, resourceID, svAllLargeData) = noErr) then begin
			if drawHighlighted then begin
				transform := ttSelected;
			end else begin
				transform := ttNone;
			end; (* if *)
			junk := PlotIconSuite(iconRect, 0, transform, iconSuite);
			junk := DisposeIconSuite(iconSuite, false);
		end else begin
			iconH := Get1Resource('ICN#', resourceID);
			if iconH <> nil then begin
				PlotIcon(iconRect, iconH);
			end; (* if *)
		end; (* if *)
	end; (* DrawIcon *)

	procedure MagicMarkerMode;
		(* See comment in interface part. *)
	begin
		if gHasColourQuickDraw then begin
			LMSetHiliteMode(band(LMGetHiliteMode, $7F));
		end; (* if *)
	end; (* MagicMarkerMode *)

	(* ***** IC API Stuff ***** *)

{$PUSH}
{$R-}
	function ICGetPrefStr (inst: ICInstance; key: Str255; var attr: ICAttr; var str: Str255): ICError;
		(* See comment in interface part. *)
		var
			err: ICError;
			size: longint;
	begin
		size := 256;
		err := ICGetPref(inst, key, attr, @str, size);
		if err <> noErr then begin
			str := '';
		end; (* if *)
		ICGetPrefStr := err;
	end; (* ICGetPrefStr *)

	function ICSetPrefStr (inst: ICInstance; key: Str255; attr: ICAttr; str: Str255): ICError;
		(* See comment in interface part. *)
	begin
		ICSetPrefStr := ICSetPref(inst, key, attr, @str, length(str) + 1);
	end; (* ICSetPrefStr *)
{$POP}

	(* ***** Text Utilities Stuff ***** *)

	function DecStr (aNumber: longint): Str255;
		(* See comment in interface part. *)
		var
			result : Str255;
	begin
		NumToString(aNumber, result);
		DecStr := result;
	end; (* DecStr *)

	function DecVal(aString : Str255) : integer;
		(* See comment in interface part. *)
		var
			result : longint;
	begin
		StringToNum(aString, result);
		DecVal := result;
	end; (* DecVal *)

	function StringToOSType (aString: Str255): OSType;
		(* See comment in interface part. *)
		var
			result: OSType;
	begin
		aString := concat(aString, chr(0), chr(0), chr(0), chr(0));
		BlockMoveData(@aString[1], @result, 4);
		StringToOSType := result;
	end; (* StringToOSType *)

	function OSTypeToString (anOSType: OSType): Str15;
		(* See comment in interface part. *)
		var
			result : Str15;
	begin
		result := concat(chr(0),chr(0),chr(0),chr(0));
		BlockMoveData(@anOSType, @result[1], 4);
		OSTypeToString := result;
	end; (* OSTypeToString *)

	function TPCopy (sourceString: string; startIndex : integer; count: integer): string;
		(* See comment in interface part. *)
	begin
		(* Check for startIndex being before the first character in the string. *)
		if startIndex < 1 then begin
			count := count - (1 - startIndex);
			startIndex := 1;
		end; (* if *)
		
		(* Check for a request for more characters than are in the string. *)
		if (startIndex + count) > length(sourceString) then begin
			count := length(sourceString) - startIndex + 1;
		end; (* if *)
		
		(* Trim count. *)
		if count < 0 then begin
			count := 0;
		end; (* if *)
		
		(* Extract the string data. *)
		sourceString[0] := chr(count);
		BlockMoveData(@sourceString[startIndex], @sourceString[1], count);

		TPCopy := sourceString;
	end; (* TPCopy *)

	function GetOwnerName : Str255;
		(* See comment in interface part. *)
		const
			rOwnerNameString = -16096;
		var
			strH: StringHandle;
	begin
		strH := GetString(rOwnerNameString);
		if strH <> nil then begin
			(* Don't release it, someone else might be using it. *)
			GetOwnerName := strH^^;
		end else begin
			GetOwnerName := '';
		end; (* if *)
	end; (* GetOwnerName *)

	function NewLookupError(NersID : integer; errNum : OSStatus) : Str255;
		(* See comment in interface part. *)
		var
			result : Str255;
			errH : Handle;
			s : SInt8;
			candidateErrNum : longint;
			errsDataPtr : BigBufferPtr;
			indexIntoErrsData : longint;
			maxIndexIntoErrsData : longint;
			found : Boolean;
	begin
		result := '';
		errH := GetResource('Ners', NersID);
		if errH <> nil then begin
			s := HGetState(errH);
			HLock(errH);
			errsDataPtr := BigBufferPtr(errH^);
			
			indexIntoErrsData := 0;
			maxIndexIntoErrsData := GetHandleSize(errH);
			found := false;
			(* Loop through the resource looking for a match. *)
			while (indexIntoErrsData < maxIndexIntoErrsData) and not found do begin
				(* Extract the error number.
					 I use BlockMoveData here because the data may not be word aligned, and
						original 68Ks will take an Address Error if I attempt to move an
						unaligned longint.
				*)
				BlockMoveData(@errsDataPtr^[indexIntoErrsData], @candidateErrNum, sizeof(longint));
				indexIntoErrsData := indexIntoErrsData + sizeof(longint);

				(* Extract the error string. *)
				BlockMoveData(@errsDataPtr^[indexIntoErrsData], @result, errsDataPtr^[indexIntoErrsData] + 1);
				indexIntoErrsData := indexIntoErrsData + errsDataPtr^[indexIntoErrsData] + 1;

				(* Figure out whether we've found what we're looking for. *)
				found := (candidateErrNum = errNum) or (candidateErrNum = 0)
			end; (* while *)
			
			if not found then begin
				result := '';
			end; (* if *)
			HSetState(errH, s);
		end; (* if *)
		NewLookupError := result;
	end; (* NewLookupError *)

	(* ***** Event Manager Stuff ***** *)

	function DirtyKey (typedChar: char): Boolean;
		(* See comment in interface part. *)
	begin
		DirtyKey := not(typedChar in [kHomeChar, kEndChar, kHelpChar, kPageUpChar, kPageDownChar,
								kLeftArrowChar, kRightArrowChar, kUpArrowChar, kDownArrowChar]);
	end; (* DirtyKey *)

	function IsKeyDown (keyCode: integer): Boolean;
		(* See comment in interface part. *)
		var
			currentKeys: KeyMap;
	begin
		GetKeys(currentKeys);
		IsKeyDown := currentKeys[keyCode];
	end; (* IsKeyDown *)

	(* ***** Window Manager Stuff ***** *)

	procedure EnterWindow (window: WindowPtr; font, size: integer; face: Style;
												var saved: SavedWindowInfo);
		(* See comment in interface part. *)
	begin
		GetPort(saved.oldPort);
		SetPort(window);
		saved.thisPort := window;
		saved.font := window^.txFont;
		saved.size := window^.txSize;
		saved.face := window^.txFace;
		TextFont(font);
		TextSize(size);
		TextFace(face);
	end; (* EnterWindow *)

	procedure ExitWindow (saved: SavedWindowInfo);
		(* See comment in interface part. *)
	begin
		SetPort(saved.thisPort);
		TextFont(saved.font);
		TextSize(saved.size);
		TextFace(saved.face);
		SetPort(saved.oldPort);
	end; (* ExitWindow *)

	function GetWindowContentRegion (theWindow: WindowPtr): RgnHandle;
		(* See comment in interface part. *)
	begin
		GetWindowContentRegion := WindowPeek(theWindow)^.contRgn;
	end; (* GetWindowContentRegion *)

	function GetWindowStructureRegion (theWindow: WindowPtr): RgnHandle;
		(* See comment in interface part. *)
	begin
		GetWindowStructureRegion := WindowPeek(theWindow)^.strucRgn;
	end; (* GetWindowStructureRegion *)

	function TitleBarOnScreen (theWindow: WindowPtr): Boolean;
		(* See comment in interface part. *)
		var
			result : Boolean;
			titleBarRegion: RgnHandle;
	begin
		result := true;
		titleBarRegion := NewRgn;
		if titleBarRegion <> nil then begin
			(* First calculate the title bar region by subtracting the content
				region away from the structure region.
			*)
			CopyRgn(GetWindowStructureRegion(theWindow), titleBarRegion);
			DiffRgn(titleBarRegion, GetWindowContentRegion(theWindow), titleBarRegion);
			
			(* Now intersect the title bar region with the grey region, ie the region
				describing the extent of the desktop and return true if the intersection
				is not empty.
			*)
			SectRgn(titleBarRegion, GetGrayRgn, titleBarRegion);
			result := not EmptyRgn(titleBarRegion);
			DisposeRgn(titleBarRegion);
		end; (* if *)
		TitleBarOnScreen := result;
	end; (* TitleBarOnScreen *)

	procedure GetWindowRect (theWindow: WindowPtr; var windowRect: Rect);
		(* See comment in interface part. *)
		var
			oldPort : GrafPtr;
	begin
		GetPort(oldPort);
		SetPort(theWindow);
		windowRect := WindowPeek(theWindow)^.port.portRect;
		LocalToGlobal(windowRect.topLeft);
		LocalToGlobal(windowRect.botRight);
		SetPort(oldPort);
	end; (* GetWindowRect *)

	(* ***** Menu Manager Stuff ***** *)

	procedure SetMenuItemEnable (menuH: MenuHandle; item: integer; enable: Boolean);
		(* See comment in interface part. *)
	begin
		if enable then begin
			EnableItem(menuH, item);
		end else begin
			DisableItem(menuH, item);
		end; (* if *)
	end; (* SetMenuItemEnable *)

	procedure SafeAppendMenu (menuH: MenuHandle; itemText: Str255);
		(* See comment in interface part. *)
	begin
		AppendMenu(menuH, 'fred');
		SetMenuItemText(menuH, CountMItems(menuH), itemText);
	end; (* SafeAppendMenu *)

	function FindMenuItem (menuH: MenuHandle; itemTextToSearchFor: Str255; 
												var indexOfItemFound: integer): Boolean;
		(* See comment in interface part. *)
		var
			itemIndex: integer;
			itemText: Str255;
	begin
		FindMenuItem := false;
		for itemIndex := 1 to CountMItems(menuH) do begin
			GetMenuItemText(menuH, itemIndex, itemText);
			if IUEqualString(itemText, itemTextToSearchFor) = 0 then begin
				indexOfItemFound := itemIndex;
				FindMenuItem := true;
			end; (* if *)
		end; (* for *)
	end; (* FindMenuItem *)

	(* ***** List Manager Stuff ***** *)

	var
		gCharsTypedSoFar: Str255;
		gTimeOfLastCharTyped: longint;
		gListHandleOfLastCharTyped: ListHandle;

	procedure InitListManagerMiscSubs;
		(* See comment in interface part. *)
	begin
		gCharsTypedSoFar := '';
		gTimeOfLastCharTyped := 0;
		gListHandleOfLastCharTyped := nil;
	end; (* InitListManagerMiscSubs *)

	procedure LSetNoSelection (listH: ListHandle);
		(* See comment in interface part. *)
		var
			listCell: Cell;
	begin
		listCell.v := 0;
		listCell.h := 0;
		while LGetSelect(true, listCell, listH) do begin
			LSetSelect(false, listCell, listH);
			listCell.v := listCell.v + 1;
			listCell.h := 0;
		end; (* if *)
	end; (* LSetNoSelection *)

	procedure LSetSingleSelection (listH: ListHandle; row: integer);
		(* See comment in interface part. *)
		var
			listCell: Cell;
	begin
		listCell.h := 0;
		listCell.v := row;
		LSetSelect(true, listCell, listH);
		listCell.v := 0;
		listCell.h := 0;
		while LGetSelect(true, listCell, listH) do begin
			if listCell.v <> row then begin
				LSetSelect(false, listCell, listH);
			end; (* if *)
			listCell.v := listCell.v + 1;
			listCell.h := 0;
		end; (* while *)
		LAutoScroll(listH);
	end; (* LSetSingleSelection *)

	function LGetUniqueEntryName (listH: ListHandle; listCell: Cell; getCellText: GetListCellTextProcType): Str255;
		(* This function calls getCellText and then returns a 'uniquified' version of the
			cell text.  What that means is that it returns the cell text followed by
			a chr(0) followed by the the vertical co-ordinate of the cell encoded
			as two characters.  This is useful because it allows functions that
			need to distinguish between two cells even if they have the same
			name to function, eg tabbing.
		*)
		var
			result: Str255;
	begin
		result := '';
		result := getCellText(listH, listCell);
		LGetUniqueEntryName := concat(result, chr(0), 
																		chr(listCell.v div 256), 
																		chr(listCell.v mod 256));
	end; (* LGetUniqueEntryName *)

	function LGetSelectedCellCommon (listH: ListHandle; var listCell: Cell; getCellText: GetListCellTextProcType;
															first : Boolean): Boolean;
		(* This function finds the alphabetically first or last cell (depending
			on the value of first) in the currently selected cells of the list.  It
			returns false if there are no selected cells.
		*)
		var
			result : Boolean;
			cellText: Str255;
			bestText : Str255;
			indexOfBestText: integer;
	begin
		(* Establish some pre-conditions. *)
		result := false;
		listCell.h := 0;
		listCell.v := 0;
		indexOfBestText := 0;
		if first then begin
			bestText := concat(chr(255), chr(255));
		end else begin
			bestText := '';
		end; (* if *)
		
		(* Loop through the selected cells, looking for the best text (ie the
			alphabetically first or last).
		*)
		while LGetSelect(true, listCell, listH) do begin
			result := true;
			cellText := getCellText(listH, listCell);
			if (first & (IUCompString(cellText, bestText) < 0)) |
						(not first & (IUCompString(cellText, bestText) > 0)) then begin
				indexOfBestText := listCell.v;
				bestText := cellText;
			end; (* if *)
			listCell.v := listCell.v + 1;
		end; (* while *)
		
		(* Finish up. *)
		listCell.h := 0;
		listCell.v := indexOfBestText;
		LGetSelectedCellCommon := result;
	end; (* LGetSelectedCellCommon *)

	function LGetFirstSelectedCell (listH: ListHandle; var listCell: Cell; getCellText: GetListCellTextProcType): Boolean;
		(* This function finds the alphabetically first cell in the currently
			selected cells of the list.  It returns false if there are no selected cells.
		*)
	begin
		LGetFirstSelectedCell := LGetSelectedCellCommon(listH, listCell, getCellText, true);
	end; (* LGetFirstSelectedCell *)

	function LGetLastSelectedCell (listH: ListHandle; var listCell: Cell; getCellText: GetListCellTextProcType): Boolean;
		(* This function finds the alphabetically last cell in the currently
			selected cells of the list.  It returns false if there are no selected cells.
		*)
	begin
		LGetLastSelectedCell := LGetSelectedCellCommon(listH, listCell, getCellText, false);
	end; (* LGetLastSelectedCell *)

	function LSelectFirstCommon (listH: ListHandle; markerText: Str255; getCellText: GetListCellTextProcType;
													before : Boolean; orEqual : Boolean): Boolean;
		(* This function selects the first cell alphabetically before (or after, depending
			on the value of "before") the markerText.  If returns true if it managed to do this,
			false otherwise.  The orEqual value determines whether an
			equal value is considered to be before the otherwise best value.
		*)
		var
			result: Boolean;
			row : integer;
			indexOfBestText: integer;
			listCell: Cell;
			bestText : Str255;
			cellText: Str255;
			comp1 : integer;
			comp2 : integer;
	begin
		(* Establish some pre-conditions. *)
		result := false;
		indexOfBestText := 0;
		if before then begin
			bestText := '';
		end else begin
			bestText := concat(chr(255), chr(255));
		end; (* if *)
		
		(* Iterate through all the cells, looking for best text.  Best is defined
			as the phone that's alphabetically before (or after, depending on
			the value of before) the markerText.
		*)
		for row := 0 to listH^^.dataBounds.bottom - 1 do begin
			listCell.h := 0;
			listCell.v := row;
			cellText := getCellText(listH, listCell);

			(* OK, so this needs some explaning (-:
				comp1 and comp2 just cache the value of the comparisons between
				markerText, cellText and bestText.
				
				If before is true, we're looking for the cell immediately before
				the markerText.  This means that the markerText must be
				greater than (ie "comp1 > 0") or equal to (ie "| (comp1 = 0)")
				the cellText, and the cellText must be greater than (ie "comp2 > 0")
				the bestText we've found so far.
				
				If before is false, we're looking for the cell immediately after
				the markerText.  This means that the markerText must be
				less than (ie "comp1 < 0") or equal to (ie "| (comp1 = 0)")
				the cellText, and the cellText must be less than (ie "comp2 < 0")
				the bestText we've found so far.
				
				*phew*
			*)
			comp1 := IUCompString(markerText, cellText);
			comp2 := IUCompString(cellText, bestText);
			if (		before & (((comp1 > 0) | ((comp1 = 0) & orEqual)) & (comp2 > 0))) |
				(not before & (((comp1 < 0) | ((comp1 = 0) & orEqual)) & (comp2 < 0))) then begin
				bestText := cellText;
				indexOfBestText := listCell.v;
				result := true;
			end; (* if *)
		end; (* for *)
		
		(* Now set the selection to the cell we found. *)
		if result then begin
			LSetSingleSelection(listH, indexOfBestText);
		end; (* if *)
		LSelectFirstCommon := result;
	end; (* LSelectFirstCommon *)

	function LSelectFirstBefore (listH: ListHandle; beforeThis: Str255; getCellText: GetListCellTextProcType): Boolean;
		(* This function selects the first cell alphabetically before
			the beforeThis text.  If returns true if it managed to do this,
			false otherwise.  The orEqual value determines whether an
			equal value is considered to be before the otherwise best value.
		*)
	begin
		LSelectFirstBefore := LSelectFirstCommon(listH, beforeThis, getCellText, true, false);
	end; (* LSelectFirstBefore *)
	
	function LSelectFirstAfter (listH: ListHandle; afterThis: Str255; getCellText: GetListCellTextProcType; orEqual:Boolean): Boolean;
		(* This function selects the first cell alphabetically after
			the afterThis text.  If returns true if it managed to do this,
			false otherwise.  The orEqual value determines whether an
			equal value is considered to be before the otherwise best value.
		*)
	begin
		LSelectFirstAfter := LSelectFirstCommon(listH, afterThis, getCellText, false, orEqual);
	end; (* LSelectFirstAfter *)

	procedure LDownArrow(listH : ListHandle);
		(* Find the last selected cell and select the cell after it. *)
		var
			listCell : Cell;
			indexOfCellToSelect : integer;
	begin
		listCell.h := 0;
		listCell.v := 0;
		indexOfCellToSelect := 0;
		while LGetSelect(true, listCell, listH) do begin
			listCell.v := listCell.v + 1;
			indexOfCellToSelect := listCell.v;
		end; (* if *)
		if indexOfCellToSelect >= listH^^.dataBounds.bottom then begin
			indexOfCellToSelect := listH^^.dataBounds.bottom - 1;
		end; (* if *)
		LSetSingleSelection(listH, indexOfCellToSelect);
		LAutoScroll(listH);
	end; (* LDownArrow *)

	procedure LUpArrow(listH : ListHandle);
		(* Find the first selected cell and select the cell before it. *)
		var
			listCell : Cell;
	begin
		listCell.h := 0;
		listCell.v := 0;
		if not LGetSelect(true, listCell, listH) then begin
			listCell.v := listH^^.dataBounds.bottom;
		end; (* if *)
		if listCell.v > 0 then begin
			listCell.v := listCell.v - 1;
		end; (* if *)
		LSetSingleSelection(listH, listCell.v);
		LAutoScroll(listH);
	end; (* LUpArrow *)
	
	procedure LTab(listH : ListHandle; getCellText: GetListCellTextProcType; shift : Boolean);
		(* Handle Tab and shift-Tab keys in the list. *)
		var
			junkBool : Boolean;
			listCell : Cell;
			done : Boolean;
			selectedCellText : Str255;
	begin
		if getCellText <> nil then begin
			if not shift then begin
				(* Tab -- If there are selected cells then attempt to select the first
					cell after the last selected cell.  If we can't or there were no 
					selected cells, then select the first cell alphabetically.
				*)
				done := false;
				if LGetLastSelectedCell(listH, listCell, getCellText) then begin
					selectedCellText := LGetUniqueEntryName(listH, listCell, getCellText);
					if LSelectFirstAfter(listH, selectedCellText, getCellText, false) then begin
						done := true;
					end; (* if *)
				end; (* if *)
				if not done then begin
					junkBool := LSelectFirstAfter(listH, '', getCellText, false);
				end; (* if *)
			end else begin
				(* shift-Tab -- If there are no selected cells then attempt to select the
					cell before the first selected cell.  If we can't or there were no 
					selected cells, then select the last cell alphabetically.
				*)
				done := false;
				if LGetFirstSelectedCell(listH, listCell, getCellText) then begin
					selectedCellText := getCellText(listH, listCell);
					if LSelectFirstBefore(listH, selectedCellText, getCellText) then begin
						done := true;
					end; (* if *)
				end; (* if *)
				if not done then begin
					junkBool := LSelectFirstBefore(listH, chr(255), getCellText);
				end; (* if *)
			end; (* if *)
		end; (* if *)
	end; (* LTab *)

	procedure LOtherKey(listH : ListHandle; getCellText : GetListCellTextProcType;
											typedChar : char; eventTicks : longint);
		(* This routine handles the pressing of a normaly key in a list
			by selecting the cell best associated with the text typed so far.
		*)
		var
			junkBool : Boolean;
	begin
		if (getCellText <> nil) & (typedChar >= ' ') then begin
			if eventTicks - gTimeOfLastCharTyped > 60 then begin
				gCharsTypedSoFar := '';
			end; (* if *)
			gTimeOfLastCharTyped := eventTicks;
			gCharsTypedSoFar := concat(gCharsTypedSoFar, typedChar);
			if not LSelectFirstAfter(listH, gCharsTypedSoFar, getCellText, true) then begin
				junkBool := LSelectFirstBefore(listH, chr(255), getCellText);
			end; (* if *)
		end; (* if *)
	end; (* LOtherKey *)

	procedure LDoKey(listH: ListHandle; var event:EventRecord; getCellText: GetListCellTextProcType);
		(* See comment in interface part. *)
		var
			eventTicks: longint;
			typedChar:char;
	begin
		eventTicks := event.when;
		typedChar := chr(band(event.message, charCodeMask));

		(* First up, if we've changed lists or typed a control character,
			we reset the globals that track the current typing state.
		*)
		if (gListHandleOfLastCharTyped <> listH) or (typedChar < ' ') then begin
			gTimeOfLastCharTyped := 0;
			gListHandleOfLastCharTyped := listH;
		end; (* if *)
		
		(* Now dispatch the various characters type. *)
		case typedChar of
			(* Handle the trivial scrolling around keys. *)
			kHomeChar:
				LScroll(0, -listH^^.dataBounds.bottom, listH);
			kEndChar:
				LScroll(0, listH^^.dataBounds.bottom, listH);
			kPageUpChar:
				LScroll(0, -(listH^^.visible.bottom - listH^^.visible.top - 2), listH);
			kPageDownChar:
				LScroll(0, (listH^^.visible.bottom - listH^^.visible.top - 2), listH);
		
			(* Handle up and down arrows. *)
			kDownArrowChar:
				LDownArrow(listH);
			kUpArrowChar:
				LUpArrow(listH);
				
			(* Tab and shift-Tab and other keys are trickier. *)
			kTabChar:
				LTab(listH, getCellText, band(event.modifiers, shiftKey) <> 0);
			otherwise
				LOtherKey(listH, getCellText, typedChar, eventTicks);
		end; (* case *)
	end; (* LDoKey *)

	function LSelectedLine (lh: ListHandle): integer;
		(* See comment in interface part. *)
		var
			listCell: Cell;
	begin
		SetPt(listCell, 0, 0);
		if LGetSelect(true, listCell, lh) then begin
			LSelectedLine := listCell.v;
		end else begin
			LSelectedLine := -1;
		end; (* if *)
	end; (* LSelectedLine *)

	(* ***** Truly Misc Stuff ***** *)

	function AEGotRequiredParams (theAppleEvent: AppleEvent): OSStatus;
		(* See comment in interface part. *)
		var
			typeCode: DescType;
			actualSize: Size;
			err: OSStatus;
	begin
		err := AEGetAttributePtr(theAppleEvent, keyMissedKeywordAttr, typeWildCard, typeCode, nil, 0, actualSize);
		if err = errAEDescNotFound then begin
			err := noErr;
		end else if err = noErr then begin
			err := errAEEventNotHandled;
		end; (* if *)
		AEGotRequiredParams := err;
	end; (* AEGotRequiredParams *)

	function NumToolboxTraps: integer;
		(* Returns the number of toolbox traps on this machine. *)
	begin
		if NGetTrapAddress(_InitGraf, ToolTrap) = NGetTrapAddress($AA6E, ToolTrap) then begin
			NumToolboxTraps := $200
		end else begin
			NumToolboxTraps := $400;
		end; (* if *)
	end; (* NumToolboxTraps *)

	function GetTrapType (theTrap: integer): TrapType;
		(* Returns the trap type associated with the given A-Trap number. *)
		const
			TrapMask = $0800;
	begin
		if band(theTrap, TrapMask) > 0 then begin
			GetTrapType := ToolTrap
		end else begin
			GetTrapType := OSTrap;
		end; (* if *)
	end; (* GetTrapType *)

	function TrapAvailable (theTrap: integer): Boolean;
		(* See comment in interface part. *)
		var
			tType: TrapType;
	begin
		tType := GetTrapType(theTrap);
		if tType = ToolTrap then begin
			theTrap := band(theTrap, $07FF);
			if theTrap >= NumToolboxTraps then begin
				theTrap := _Unimplemented;
			end; (* if *)
		end; (* if *)
		TrapAvailable := NGetTrapAddress(theTrap, tType) <> NGetTrapAddress(_Unimplemented, ToolTrap);
	end; (* TrapAvailable *)

end. (* ICMiscSubs *)
